@@ -4,6 +4,22 @@
 #include <libgda-ui/libgda-ui.h>
 #include <glib-object.h>
 
+/* enumerations from "gdaui-basic-form.h" */
+GType
+gdaui_basic_form_part_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GDAUI_BASIC_FORM_LABELS, "GDAUI_BASIC_FORM_LABELS", "labels" },
+      { GDAUI_BASIC_FORM_ENTRIES, "GDAUI_BASIC_FORM_ENTRIES", "entries" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GdauiBasicFormPart", values);
+  }
+  return etype;
+}
+
 /* enumerations from "gdaui-data-proxy.h" */
 GType
 gdaui_data_proxy_write_mode_get_type (void)
