@@ -23,7 +23,7 @@
 #ifndef __GDAUI_PROVIDER_SPEC_EDITOR_H__
 #define __GDAUI_PROVIDER_SPEC_EDITOR_H__
 
-#include <gtk/gtk.h>
+#include <libgda-ui/gdaui-basic-form.h>
 
 G_BEGIN_DECLS
 
@@ -57,6 +57,9 @@ gboolean    _gdaui_provider_spec_editor_is_valid     (GdauiProviderSpecEditor *s
 gchar      *_gdaui_provider_spec_editor_get_specs    (GdauiProviderSpecEditor *spec);
 void        _gdaui_provider_spec_editor_set_specs    (GdauiProviderSpecEditor *spec, const gchar *specs_string);
 
+void        _gdaui_provider_spec_editor_add_to_size_group (GdauiProviderSpecEditor *spec, GtkSizeGroup *size_group,
+							   GdauiBasicFormPart part);
+
 G_END_DECLS
 
 #endif
