@@ -255,7 +255,8 @@ _gdaui_utility_proxy_compute_attributes_for_group (GdauiSetGroup *group, GdauiDa
 }
 
 /**
- * _gdaui_utility_proxy_compute_values_for_group
+ * _gdaui_utility_proxy_compute_values_for_group:
+ *
  * Computes a list of values containing the individual values stored in @store and
  * corresponding to the columns for the parameters if @group, at row pointed by @tree_iter.
  * 
@@ -270,8 +271,8 @@ _gdaui_utility_proxy_compute_attributes_for_group (GdauiSetGroup *group, GdauiDa
  */
 GList *
 _gdaui_utility_proxy_compute_values_for_group (GdauiSetGroup *group, GdauiDataStore *store, 
-						 GdaDataModelIter *model_iter, 
-						 GtkTreeIter *tree_iter, gboolean model_values)
+					       GdaDataModelIter *model_iter, 
+					       GtkTreeIter *tree_iter, gboolean model_values)
 {
 	gint offset;
 	GList *retval = NULL;
@@ -384,7 +385,7 @@ _gdaui_utility_proxy_compute_values_for_group (GdauiSetGroup *group, GdauiDataSt
  * Errors reporting
  */
 static GtkWidget *
-create_data_error_dialog (GdauiDataWidget *form, gboolean with_question, gboolean can_discard, GError *filled_error)
+create_data_error_dialog (GdauiDataProxy *form, gboolean with_question, gboolean can_discard, GError *filled_error)
 {
 	GtkWidget *dlg;
 	const gchar *msg1 = NULL, *msg2 = NULL;
@@ -439,7 +440,12 @@ create_data_error_dialog (GdauiDataWidget *form, gboolean with_question, gboolea
 		
 		gtk_container_add (GTK_CONTAINER (sw), view);
 		gtk_container_add (GTK_CONTAINER (exp), sw);
+#if GTK_CHECK_VERSION(2,18,0)
+		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
+				    exp, TRUE, TRUE, 0);
+#else
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), exp, TRUE, TRUE, 0);
+#endif
 		gtk_widget_show_all (exp);
 
 		gtk_window_set_resizable (GTK_WINDOW (dlg), TRUE);
@@ -450,7 +456,7 @@ create_data_error_dialog (GdauiDataWidget *form, gboolean with_question, gboolea
 
 /**
  * _gdaui_utility_display_error_with_keep_or_discard_choice
- * @form:
+ * @form: a #GdauiDataProxy
  * @filled_error: a #GError containing the error to display
  *
  * Displays a dialog showing @filled_error's message and asks the user to either keep the data which
@@ -459,7 +465,7 @@ create_data_error_dialog (GdauiDataWidget *form, gboolean with_question, gboolea
  * Returns: TRUE if current data can be discarded
  */
 gboolean
-_gdaui_utility_display_error_with_keep_or_discard_choice (GdauiDataWidget *form, GError *filled_error)
+_gdaui_utility_display_error_with_keep_or_discard_choice (GdauiDataProxy *form, GError *filled_error)
 {
 	GtkWidget *dlg;
 	gint res;
@@ -479,14 +485,14 @@ _gdaui_utility_display_error_with_keep_or_discard_choice (GdauiDataWidget *form,
 
 /**
  * _gdaui_utility_display_error
- * @form:
+ * @form: a #GdauiDataProxy
  * @filled_error: a #GError containing the error to display
  *
  * Displays a dialog showing @filled_error's message and asks the user to either keep the data which
  * led to the error.
  */
 void
-_gdaui_utility_display_error (GdauiDataWidget *form, gboolean can_discard, GError *filled_error)
+_gdaui_utility_display_error (GdauiDataProxy *form, gboolean can_discard, GError *filled_error)
 {
 	GtkWidget *dlg;
 	
@@ -533,3 +539,48 @@ _gdaui_utility_show_error (GtkWindow *parent, const gchar *format, ...)
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 }
+
+/*
+ * Compares 2 GdaDataModelIter
+ *
+ * Returns: %TRUE if they have at least one difference
+ */
+gboolean
+_gdaui_utility_iter_differ (GdaDataModelIter *iter1, GdaDataModelIter *iter2)
+{
+	GSList *list1, *list2;
+        gboolean retval = TRUE;
+
+	for (list1 = GDA_SET (iter1)->holders, list2 = GDA_SET (iter2)->holders;
+             list1 && list2;
+             list1 = list1->next, list2 = list2->next) {
+                GdaHolder *oh, *nh;
+                oh = GDA_HOLDER (list1->data);
+                nh = GDA_HOLDER (list2->data);
+
+                if (gda_holder_get_not_null (oh) != gda_holder_get_not_null (nh))
+                        goto out;
+
+                GType ot, nt;
+                ot = gda_holder_get_g_type (oh);
+                nt = gda_holder_get_g_type (nh);
+                if (ot && (ot != nt))
+                        goto out;
+                if (ot != nt)
+                        g_object_set (oh, "g-type", nt, NULL);
+
+                const gchar *oid, *nid;
+                oid = gda_holder_get_id (oh);
+                nid = gda_holder_get_id (nh);
+                if ((oid && !nid) || (!oid && nid) ||
+                    (oid && strcmp (oid, nid)))
+                        goto out;
+        }
+
+        if (list1 || list2)
+                goto out;
+        retval = FALSE; /* iters don't differ */
+
+ out:
+        return retval;
+}
