@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 2007 - 2008 Vivien Malerba
+/*
+ * Copyright (C) 2007 - 2009 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -25,13 +25,22 @@
 #include <sql-parser/gda-statement-struct-decl.h>
 #include <libgda/gda-meta-store.h>
 
+G_BEGIN_DECLS
+
 struct _GdaSqlStatement {
 	gchar               *sql;
 	GdaSqlStatementType  stmt_type;
 	gpointer             contents; /* depends on stmt_type */
 	GdaMetaStruct       *validity_meta_struct; /* set when gda_sql_statement_check_validity() was last called */
+
+	/* Padding for future expansion */
+	gpointer         _gda_reserved1;
+	gpointer         _gda_reserved2;
 };
 
+#define GDA_TYPE_SQL_STATEMENT (gda_sql_statement_get_type())
+
+GType                        gda_sql_statement_get_type  (void) G_GNUC_CONST;
 GdaSqlStatement             *gda_sql_statement_new       (GdaSqlStatementType type);
 GdaSqlStatement             *gda_sql_statement_copy      (GdaSqlStatement *stmt);
 void                         gda_sql_statement_free      (GdaSqlStatement *stmt);
@@ -47,4 +56,6 @@ gboolean                     gda_sql_statement_normalize       (GdaSqlStatement
 
 GdaSqlStatementContentsInfo *gda_sql_statement_get_contents_infos (GdaSqlStatementType type) ;
 
+G_END_DECLS
+
 #endif
