@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Vivien Malerba
+ * Copyright (C) 2009 - 2010 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -31,6 +31,9 @@
 /* code inclusion */
 #include "../dict-file-name.c"
 
+#define CHECK_RESULTS_SHORT_TIMER 200
+#define CHECK_RESULTS_LONG_TIMER 2
+
 typedef struct {
 	GObject *result;
 	GError  *error;
@@ -89,10 +92,21 @@ static void
 push_wrapper_job (BrowserConnection *bcnc, guint job_id, JobType job_type, const gchar *reason)
 {
 	/* setup timer */
-	if (bcnc->priv->wrapper_results_timer == 0)
-		bcnc->priv->wrapper_results_timer = g_timeout_add (200,
+	if (bcnc->priv->wrapper_results_timer == 0) {
+		bcnc->priv->long_timer = FALSE;
+		bcnc->priv->wrapper_results_timer = g_timeout_add (CHECK_RESULTS_SHORT_TIMER,
 								   (GSourceFunc) check_for_wrapper_result,
 								   bcnc);
+	}
+	else if (bcnc->priv->long_timer) {
+		/* switch to a short timer to check for results */
+		g_source_remove (bcnc->priv->wrapper_results_timer);
+		bcnc->priv->wrapper_results_timer = g_timeout_add (CHECK_RESULTS_SHORT_TIMER,
+								   (GSourceFunc) check_for_wrapper_result,
+								   bcnc);
+		bcnc->priv->long_timer = FALSE;
+		bcnc->priv->nb_no_job_waits = 0;
+	}
 
 	/* add WrapperJob structure */
 	WrapperJob *wj;
@@ -241,6 +255,8 @@ browser_connection_init (BrowserConnection *bcnc)
 	bcnc->priv->wrapper = gda_thread_wrapper_new ();
 	bcnc->priv->wrapper_jobs = NULL;
 	bcnc->priv->wrapper_results_timer = 0;
+	bcnc->priv->long_timer = FALSE;
+	bcnc->priv->nb_no_job_waits = 0;
 	bcnc->priv->executed_statements = NULL;
 
 	bcnc->priv->name = g_strdup_printf (_("c%u"), index++);
@@ -249,9 +265,12 @@ browser_connection_init (BrowserConnection *bcnc)
 	bcnc->priv->variables = NULL;
 	memset (&(bcnc->priv->dsn_info), 0, sizeof (GdaDsnInfo));
 	bcnc->priv->p_mstruct_mutex = g_mutex_new ();
-	bcnc->priv->p_mstruct = NULL;
+	bcnc->priv->p_mstruct_list = NULL;
+	bcnc->priv->c_mstruct = NULL;
 	bcnc->priv->mstruct = NULL;
 
+	bcnc->priv->meta_store_signal = 0;
+
 	bcnc->priv->bfav = NULL;
 
 	bcnc->priv->store_cnc = NULL;
@@ -260,7 +279,9 @@ browser_connection_init (BrowserConnection *bcnc)
 }
 
 static void
-transaction_status_changed_cb (GdaConnection *cnc, BrowserConnection *bcnc)
+transaction_status_changed_cb (GdaThreadWrapper *wrapper, gpointer instance, const gchar *signame,
+			       gint n_param_values, const GValue *param_values, gpointer gda_reserved,
+			       BrowserConnection *bcnc)
 {
 	g_signal_emit (bcnc, browser_connection_signals [TRANSACTION_STATUS_CHANGED], 0);
 }
@@ -280,18 +301,60 @@ wrapper_meta_store_update (BrowserConnection *bcnc, GError **error)
 static gpointer
 wrapper_meta_struct_sync (BrowserConnection *bcnc, GError **error)
 {
-	gboolean retval;
+	gboolean retval = TRUE;
+	GdaMetaStruct *mstruct;
 
 	g_mutex_lock (bcnc->priv->p_mstruct_mutex);
-	g_assert (bcnc->priv->p_mstruct);
-	g_object_ref (G_OBJECT (bcnc->priv->p_mstruct));
-	retval = gda_meta_struct_complement_all (bcnc->priv->p_mstruct, error);
-	g_object_unref (G_OBJECT (bcnc->priv->p_mstruct));
+	g_assert (bcnc->priv->p_mstruct_list);
+	mstruct = (GdaMetaStruct *) bcnc->priv->p_mstruct_list->data;
+	bcnc->priv->p_mstruct_list = g_slist_delete_link (bcnc->priv->p_mstruct_list,
+							  bcnc->priv->p_mstruct_list);
+	if (bcnc->priv->p_mstruct_list)
+		/* don't care about this one */
+		g_object_unref (G_OBJECT (mstruct));
+	else {
+		if (bcnc->priv->c_mstruct)
+			g_object_unref (bcnc->priv->c_mstruct);
+		bcnc->priv->c_mstruct = mstruct;
+		/*g_print ("Meta struct sync for %p\n", mstruct);*/
+		retval = gda_meta_struct_complement_all (mstruct, error);
+	}
 	g_mutex_unlock (bcnc->priv->p_mstruct_mutex);
 
 	return GINT_TO_POINTER (retval ? 2 : 1);
 }
 
+static void
+meta_changed_cb (GdaThreadWrapper *wrapper,
+		 GdaMetaStore *store,
+		 const gchar *signame,
+		 gint n_param_values,
+		 const GValue *param_values,
+		 gpointer gda_reserved,
+		 BrowserConnection *bcnc)
+{
+	guint job_id;
+	GError *lerror = NULL;
+	GdaMetaStruct *mstruct;
+	
+	g_mutex_lock (bcnc->priv->p_mstruct_mutex);
+	mstruct = gda_meta_struct_new (gda_connection_get_meta_store (bcnc->priv->cnc),
+				       GDA_META_STRUCT_FEATURE_ALL);
+	bcnc->priv->p_mstruct_list = g_slist_append (bcnc->priv->p_mstruct_list, mstruct);
+	g_mutex_unlock (bcnc->priv->p_mstruct_mutex);
+	job_id = gda_thread_wrapper_execute (bcnc->priv->wrapper,
+					     (GdaThreadWrapperFunc) wrapper_meta_struct_sync,
+					     g_object_ref (bcnc), g_object_unref, &lerror);
+	if (job_id > 0)
+		push_wrapper_job (bcnc, job_id, JOB_TYPE_META_STRUCT_SYNC,
+				  _("Analysing database schema"));
+	else if (lerror) {
+		browser_show_error (NULL, _("Error while fetching meta data from the connection: %s"),
+				    lerror->message ? lerror->message : _("No detail"));
+		g_error_free (lerror);
+	}
+}
+
 static void
 browser_connection_set_property (GObject *object,
 				 guint param_id,
@@ -309,8 +372,14 @@ browser_connection_set_property (GObject *object,
 				return;
 
 			g_object_ref (bcnc->priv->cnc);
-			g_signal_connect (bcnc->priv->cnc, "transaction-status-changed",
-					  G_CALLBACK (transaction_status_changed_cb), bcnc);
+			bcnc->priv->transaction_status_signal =
+				gda_thread_wrapper_connect_raw (bcnc->priv->wrapper,
+								bcnc->priv->cnc,
+								"transaction-status-changed",
+								FALSE, FALSE,
+								(GdaThreadWrapperCallback) transaction_status_changed_cb,
+								bcnc);
+
 
 			/* meta store */
 			gchar *dict_file_name = NULL;
@@ -354,8 +423,11 @@ browser_connection_set_property (GObject *object,
 			else {
 				guint job_id;
 				GError *lerror = NULL;
-				
-				bcnc->priv->p_mstruct = gda_meta_struct_new (store, GDA_META_STRUCT_FEATURE_ALL);
+				GdaMetaStruct *mstruct;
+
+				mstruct = gda_meta_struct_new (store, GDA_META_STRUCT_FEATURE_ALL);
+				bcnc->priv->p_mstruct_list = g_slist_append (bcnc->priv->p_mstruct_list,
+									     mstruct);
 				job_id = gda_thread_wrapper_execute (bcnc->priv->wrapper,
 								     (GdaThreadWrapperFunc) wrapper_meta_struct_sync,
 								     g_object_ref (bcnc), g_object_unref, &lerror);
@@ -369,11 +441,18 @@ browser_connection_set_property (GObject *object,
 				}
 				g_object_unref (store);
 			}
+			bcnc->priv->meta_store_signal =
+				gda_thread_wrapper_connect_raw (bcnc->priv->wrapper, store, "meta-changed",
+								FALSE, FALSE,
+								(GdaThreadWrapperCallback) meta_changed_cb,
+								bcnc);
                         break;
                 }
         }
 }
 
+
+
 static void
 browser_connection_get_property (GObject *object,
 				 guint param_id,
@@ -427,6 +506,16 @@ browser_connection_dispose (GObject *object)
 
 	bcnc = BROWSER_CONNECTION (object);
 	if (bcnc->priv) {
+		if (bcnc->priv->results_timer_id) {
+			g_source_remove (bcnc->priv->results_timer_id);
+			bcnc->priv->results_timer_id = 0;
+		}
+		if (bcnc->priv->results_list) {
+			g_slist_foreach (bcnc->priv->results_list, (GFunc) g_free, NULL);
+			g_slist_free (bcnc->priv->results_list);
+			bcnc->priv->results_list = NULL;
+		}
+
 		if (bcnc->priv->variables)
 			g_object_unref (bcnc->priv->variables);
 
@@ -448,20 +537,28 @@ browser_connection_dispose (GObject *object)
 		if (bcnc->priv->wrapper_results_timer > 0)
 			g_source_remove (bcnc->priv->wrapper_results_timer);
 
+		if (bcnc->priv->meta_store_signal)
+			gda_thread_wrapper_disconnect (bcnc->priv->wrapper,
+						       bcnc->priv->meta_store_signal);
 		g_object_unref (bcnc->priv->wrapper);
 		g_free (bcnc->priv->name);
+		if (bcnc->priv->c_mstruct)
+			g_object_unref (bcnc->priv->c_mstruct);
 		if (bcnc->priv->mstruct)
 			g_object_unref (bcnc->priv->mstruct);
-		if (bcnc->priv->p_mstruct)
-			g_object_unref (bcnc->priv->p_mstruct);
+		if (bcnc->priv->p_mstruct_list) {
+			g_slist_foreach (bcnc->priv->p_mstruct_list, (GFunc) g_object_unref, NULL);
+			g_slist_free (bcnc->priv->p_mstruct_list);
+		}
 		if (bcnc->priv->p_mstruct_mutex)
 			g_mutex_free (bcnc->priv->p_mstruct_mutex);
-		if (bcnc->priv->cnc) {
-			g_signal_handlers_disconnect_by_func (bcnc->priv->cnc,
-							      G_CALLBACK (transaction_status_changed_cb),
-							      bcnc);
+
+		if (bcnc->priv->transaction_status_signal)
+			gda_thread_wrapper_disconnect (bcnc->priv->wrapper,
+						       bcnc->priv->transaction_status_signal);
+		if (bcnc->priv->cnc)
 			g_object_unref (bcnc->priv->cnc);
-		}
+
 		if (bcnc->priv->parser)
 			g_object_unref (bcnc->priv->parser);
 		if (bcnc->priv->bfav) {
@@ -485,9 +582,34 @@ check_for_wrapper_result (BrowserConnection *bcnc)
 	GError *lerror = NULL;
 	gpointer exec_res = NULL;
 	WrapperJob *wj;
+	gboolean retval = TRUE; /* return FALSE to interrupt current timer */
 
-	if (!bcnc->priv->wrapper_jobs)
-		goto out;
+	if (!bcnc->priv->wrapper_jobs) {
+		gda_thread_wrapper_iterate (bcnc->priv->wrapper, FALSE);
+		if (! bcnc->priv->long_timer) {
+			if (bcnc->priv->nb_no_job_waits > 100) {
+				/* switch to a long timer to check for results */
+				bcnc->priv->wrapper_results_timer = g_timeout_add_seconds (CHECK_RESULTS_LONG_TIMER,
+											   (GSourceFunc) check_for_wrapper_result,
+											   bcnc);
+				bcnc->priv->nb_no_job_waits = 0;
+				bcnc->priv->long_timer = TRUE;
+				return FALSE;
+			}
+			else
+				bcnc->priv->nb_no_job_waits ++;
+		}
+		return TRUE;
+	}
+	else if (bcnc->priv->long_timer) {
+		/* switch to a short timer to check for results */
+		bcnc->priv->wrapper_results_timer = g_timeout_add (CHECK_RESULTS_SHORT_TIMER,
+								   (GSourceFunc) check_for_wrapper_result,
+								   bcnc);
+		retval = FALSE;
+		bcnc->priv->long_timer = FALSE;
+		bcnc->priv->nb_no_job_waits = 0;
+	}
 
 	wj = (WrapperJob*) bcnc->priv->wrapper_jobs->data;
 	exec_res = gda_thread_wrapper_fetch_result (bcnc->priv->wrapper,
@@ -501,26 +623,17 @@ check_for_wrapper_result (BrowserConnection *bcnc)
 						    lerror && lerror->message ? lerror->message : _("No detail"));
 				g_clear_error (&lerror);
 			}
-			else {
-				guint job_id;
-				
-				g_mutex_lock (bcnc->priv->p_mstruct_mutex);
-				if (bcnc->priv->p_mstruct)
-					g_object_unref (G_OBJECT (bcnc->priv->p_mstruct));
-				bcnc->priv->p_mstruct = gda_meta_struct_new (gda_connection_get_meta_store (bcnc->priv->cnc),
-									     GDA_META_STRUCT_FEATURE_ALL);
-				g_mutex_unlock (bcnc->priv->p_mstruct_mutex);
-				job_id = gda_thread_wrapper_execute (bcnc->priv->wrapper,
-								     (GdaThreadWrapperFunc) wrapper_meta_struct_sync,
-								     g_object_ref (bcnc), g_object_unref, &lerror);
-				if (job_id > 0)
-					push_wrapper_job (bcnc, job_id, JOB_TYPE_META_STRUCT_SYNC,
-							  _("Analysing database schema"));
-				else if (lerror) {
-					browser_show_error (NULL, _("Error while fetching meta data from the connection: %s"),
-							    lerror->message ? lerror->message : _("No detail"));
-					g_error_free (lerror);
-				}
+			else if (! bcnc->priv->meta_store_signal) {
+				GdaMetaStore *store;
+				store = gda_connection_get_meta_store (bcnc->priv->cnc);
+				meta_changed_cb (bcnc->priv->wrapper, store,
+						 NULL, 0, NULL, NULL, bcnc);
+				bcnc->priv->meta_store_signal =
+					gda_thread_wrapper_connect_raw (bcnc->priv->wrapper, store, "meta-changed",
+									FALSE, FALSE,
+									(GdaThreadWrapperCallback) meta_changed_cb,
+									bcnc);
+
 			}
 			break;
 		}
@@ -532,24 +645,27 @@ check_for_wrapper_result (BrowserConnection *bcnc)
 			}
 			else {
 				g_mutex_lock (bcnc->priv->p_mstruct_mutex);
-				GdaMetaStruct *old_mstruct;
-				old_mstruct = bcnc->priv->mstruct;
-				bcnc->priv->mstruct = bcnc->priv->p_mstruct;
-				bcnc->priv->p_mstruct = NULL;
-				if (old_mstruct)
-					g_object_unref (old_mstruct);
+				
+				if (bcnc->priv->c_mstruct) {
+					GdaMetaStruct *old_mstruct;
+					old_mstruct = bcnc->priv->mstruct;
+					bcnc->priv->mstruct = bcnc->priv->c_mstruct;
+					bcnc->priv->c_mstruct = NULL;
+					if (old_mstruct)
+						g_object_unref (old_mstruct);
 #ifdef GDA_DEBUG_NO
-				GSList *all, *list;
-				all = gda_meta_struct_get_all_db_objects (bcnc->priv->mstruct);
-				for (list = all; list; list = list->next) {
-					GdaMetaDbObject *dbo = (GdaMetaDbObject *) list->data;
-					g_print ("DBO, Type %d: short=>[%s] schema=>[%s] full=>[%s]\n", dbo->obj_type,
-						 dbo->obj_short_name, dbo->obj_schema, dbo->obj_full_name);
-				}
-				g_slist_free (all);
+					GSList *all, *list;
+					g_print ("For GdaMetaStruct %p:\n", bcnc->priv->mstruct);
+					all = gda_meta_struct_get_all_db_objects (bcnc->priv->mstruct);
+					for (list = all; list; list = list->next) {
+						GdaMetaDbObject *dbo = (GdaMetaDbObject *) list->data;
+						g_print ("DBO, Type %d: short=>[%s] schema=>[%s] full=>[%s]\n", dbo->obj_type,
+							 dbo->obj_short_name, dbo->obj_schema, dbo->obj_full_name);
+					}
+					g_slist_free (all);
 #endif
-
-				g_signal_emit (bcnc, browser_connection_signals [META_CHANGED], 0, bcnc->priv->mstruct);
+					g_signal_emit (bcnc, browser_connection_signals [META_CHANGED], 0, bcnc->priv->mstruct);
+				}
 				g_mutex_unlock (bcnc->priv->p_mstruct_mutex);
 			}
 			break;
@@ -581,17 +697,12 @@ check_for_wrapper_result (BrowserConnection *bcnc)
 		pop_wrapper_job (bcnc, wj);
 	}
 
- out:
-	if (!bcnc->priv->wrapper_jobs) {
-		bcnc->priv->wrapper_results_timer = 0;
-		return FALSE;
-	}
-	else {
+	if (bcnc->priv->wrapper_jobs) {
 		wj = (WrapperJob*) bcnc->priv->wrapper_jobs->data;
 		if (exec_res)
 			g_signal_emit (bcnc, browser_connection_signals [BUSY], 0, TRUE, wj->reason);
-		return TRUE;
 	}
+	return retval;
 }
 
 /**
@@ -721,6 +832,12 @@ browser_connection_update_meta_data (BrowserConnection *bcnc)
 		}
 	}
 
+	if (bcnc->priv->meta_store_signal) {
+		gda_thread_wrapper_disconnect (bcnc->priv->wrapper,
+					       bcnc->priv->meta_store_signal);
+		bcnc->priv->meta_store_signal = 0;
+	}
+
 	guint job_id;
 	GError *lerror = NULL;
 	job_id = gda_thread_wrapper_execute (bcnc->priv->wrapper,
@@ -951,7 +1068,8 @@ wrapper_statement_execute (StmtExecData *data, GError **error)
  * @need_last_insert_row: %TRUE if the values of the last interted row must be computed
  * @error: a place to store errors, or %NULL
  *
- * Executes @stmt by @bcnc
+ * Executes @stmt by @bcnc. Unless specific requirements, it's easier to use
+ * browser_connection_execute_statement_cb().
  *
  * Returns: a job ID, to be used with browser_connection_execution_get_result(), or %0 if an
  * error occurred
@@ -1005,6 +1123,13 @@ wrapper_rerun_select (RerunSelectData *data, GError **error)
 
 /**
  * browser_connection_rerun_select
+ * @bcnc: a #BrowserConnection object
+ * @model: a #GdaDataModel, which has to ba a #GdaDataSelect
+ * @error: a place to store errors, or %NULL
+ *
+ * Re-execute @model
+ *
+ * Returns: a job ID, or %0 if an error occurred
  */
 guint
 browser_connection_rerun_select (BrowserConnection *bcnc,
@@ -1076,9 +1201,159 @@ browser_connection_execution_get_result (BrowserConnection *bcnc, guint exec_id,
 	}
 
 	g_hash_table_remove (bcnc->priv->executed_statements, &id);
+	/*if (GDA_IS_DATA_MODEL (retval))
+	  gda_data_model_dump (GDA_DATA_MODEL (retval), NULL);*/
 	return retval;
 }
 
+static gboolean query_exec_fetch_cb (BrowserConnection *bcnc);
+
+typedef struct {
+	guint exec_id;
+	gboolean need_last_insert_row;
+	BrowserConnectionExecuteCallback callback;
+	gpointer cb_data;
+} ExecCallbackData;
+
+/**
+ * browser_connection_execute_statement_cb
+ * @bcnc: a #BrowserConnection
+ * @stmt: a #GdaStatement
+ * @params: a #GdaSet as parameters, or %NULL
+ * @model_usage: how the returned data model (if any) will be used
+ * @need_last_insert_row: %TRUE if the values of the last interted row must be computed
+ * @callback: the function to call when statement has been executed
+ * @data: data to pass to @callback, or %NULL
+ * @error: a place to store errors, or %NULL
+ *
+ * Executes @stmt by @bcnc and calls @callback when done. This occurs in the UI thread and avoids
+ * having to set up a waiting mechanism to call browser_connection_execution_get_result()
+ * repeatedly.
+ *
+ * Returns: a job ID, or %0 if an error occurred
+ */
+guint
+browser_connection_execute_statement_cb (BrowserConnection *bcnc,
+					 GdaStatement *stmt,
+					 GdaSet *params,
+					 GdaStatementModelUsage model_usage,
+					 gboolean need_last_insert_row,
+					 BrowserConnectionExecuteCallback callback,
+					 gpointer data,
+					 GError **error)
+{
+	guint exec_id;
+	g_return_val_if_fail (callback, 0);
+
+	exec_id = browser_connection_execute_statement (bcnc, stmt, params, model_usage,
+							need_last_insert_row, error);
+	if (!exec_id)
+		return 0;
+	ExecCallbackData *cbdata;
+	cbdata = g_new0 (ExecCallbackData, 1);
+	cbdata->exec_id = exec_id;
+	cbdata->need_last_insert_row = need_last_insert_row;
+	cbdata->callback = callback;
+	cbdata->cb_data = data;
+
+	bcnc->priv->results_list = g_slist_append (bcnc->priv->results_list, cbdata);
+	if (! bcnc->priv->results_timer_id)
+		bcnc->priv->results_timer_id = g_timeout_add (200,
+							      (GSourceFunc) query_exec_fetch_cb,
+							      bcnc);
+	return exec_id;
+}
+
+/**
+ * browser_connection_rerun_select_cb
+ * @bcnc: a #BrowserConnection object
+ * @model: a #GdaDataModel, which has to ba a #GdaDataSelect
+ * @callback: the function to call when statement has been executed
+ * @data: data to pass to @callback, or %NULL
+ * @error: a place to store errors, or %NULL
+ *
+ * Re-execute @model.
+ *
+ * Warning: gda_data_model_freeze() and gda_data_model_thaw() should be used
+ * before and after this call since the model will signal its changes in a thread
+ * which is not the GUI thread.
+ *
+ * Returns: a job ID, or %0 if an error occurred
+ */
+guint
+browser_connection_rerun_select_cb (BrowserConnection *bcnc,
+				    GdaDataModel *model,
+				    BrowserConnectionExecuteCallback callback,
+				    gpointer data,
+				    GError **error)
+{
+	guint exec_id;
+	g_return_val_if_fail (callback, 0);
+
+	exec_id = browser_connection_rerun_select (bcnc, model, error);
+	if (!exec_id)
+		return 0;
+	ExecCallbackData *cbdata;
+	cbdata = g_new0 (ExecCallbackData, 1);
+	cbdata->exec_id = exec_id;
+	cbdata->need_last_insert_row = FALSE;
+	cbdata->callback = callback;
+	cbdata->cb_data = data;
+
+	bcnc->priv->results_list = g_slist_append (bcnc->priv->results_list, cbdata);
+	if (! bcnc->priv->results_timer_id)
+		bcnc->priv->results_timer_id = g_timeout_add (200,
+							      (GSourceFunc) query_exec_fetch_cb,
+							      bcnc);
+	return exec_id;
+}
+
+
+static gboolean
+query_exec_fetch_cb (BrowserConnection *bcnc)
+{
+	GObject *res;
+	GError *lerror = NULL;
+	ExecCallbackData *cbdata;
+	GdaSet *last_inserted_row = NULL;
+
+	if (!bcnc->priv->results_list)
+		goto out;
+
+	cbdata = (ExecCallbackData *) bcnc->priv->results_list->data;
+
+	if (cbdata->need_last_insert_row)
+		res = browser_connection_execution_get_result (bcnc,
+							       cbdata->exec_id,
+							       &last_inserted_row,
+							       &lerror);
+	else
+		res = browser_connection_execution_get_result (bcnc,
+							       cbdata->exec_id, NULL,
+							       &lerror);
+
+	if (res || lerror) {
+		cbdata->callback (bcnc, cbdata->exec_id, res, last_inserted_row, lerror, cbdata->cb_data);
+		if (res)
+			g_object_unref (res);
+		if (last_inserted_row)
+			g_object_unref (last_inserted_row);
+		g_clear_error (&lerror);
+
+		bcnc->priv->results_list = g_slist_remove (bcnc->priv->results_list, cbdata);
+		g_free (cbdata);
+	}
+
+ out:
+	if (! bcnc->priv->results_list) {
+		bcnc->priv->results_timer_id = 0;
+		return FALSE;
+	}
+	else
+		return TRUE; /* keep timer */
+}
+
+
 /**
  * browser_connection_normalize_sql_statement
  * @bcnc: a #BrowserConnection
@@ -1308,7 +1583,7 @@ browser_connection_set_table_column_attribute (BrowserConnection *bcnc,
  * @error:
  *
  *
- * Returns: the requested attribute, or %NULL if not set or if an error occurred
+ * Returns: the requested attribute (as a new string), or %NULL if not set or if an error occurred
  */
 gchar *
 browser_connection_get_table_column_attribute  (BrowserConnection *bcnc,
