@@ -30,6 +30,7 @@
 #include "browser-spinner.h"
 #include "browser-stock-icons.h"
 #include "connection-binding-properties.h"
+#include <gdk/gdkkeysyms.h>
 
 /*
  * structure representing a 'tab' in a window
@@ -59,6 +60,7 @@ static void browser_window_init (BrowserWindow *bwin);
 static void browser_window_dispose (GObject *object);
 
 static gboolean window_state_event (GtkWidget *widget, GdkEventWindowState *event);
+static gboolean key_press_event (GtkWidget *widget, GdkEventKey *event);
 static void connection_busy_cb (BrowserConnection *bcnc, gboolean is_busy, gchar *reason, BrowserWindow *bwin);
 
 static void connection_added_cb (BrowserCore *bcore, BrowserConnection *bcnc, BrowserWindow *bwin);
@@ -67,6 +69,14 @@ static void connection_removed_cb (BrowserCore *bcore, BrowserConnection *bcnc,
 static void transaction_status_changed_cb (BrowserConnection *bcnc, BrowserWindow *bwin);
 
 
+enum {
+        FULLSCREEN_CHANGED,
+        LAST_SIGNAL
+};
+
+static gint browser_window_signals[LAST_SIGNAL] = { 0 };
+
+
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass  *parent_class = NULL;
 
@@ -90,6 +100,8 @@ struct _BrowserWindowPrivate {
 
 	GtkWidget         *statusbar;
 	guint              cnc_statusbar_context;
+
+	gboolean           fullscreen;
 };
 
 GType
@@ -125,11 +137,22 @@ browser_window_class_init (BrowserWindowClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
-	widget_class->window_state_event = window_state_event;
 
+	widget_class->window_state_event = window_state_event;
+	widget_class->key_press_event = key_press_event;
 	parent_class = g_type_class_peek_parent (klass);
 
+	browser_window_signals[FULLSCREEN_CHANGED] =
+                g_signal_new ("fullscreen_changed",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (BrowserWindowClass, fullscreen_changed),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__BOOLEAN,
+                              G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
+
 	object_class->dispose = browser_window_dispose;
+	klass->fullscreen_changed = NULL;
 }
 
 static void
@@ -143,6 +166,7 @@ browser_window_init (BrowserWindow *bwin)
 	bwin->priv->cnc_added_sigid = 0;
 	bwin->priv->cnc_removed_sigid = 0;
 	bwin->priv->updating_transaction_status = FALSE;
+	bwin->priv->fullscreen = FALSE;
 }
 
 static void
@@ -235,6 +259,7 @@ static const GtkActionEntry ui_actions[] = {
         { "ConnectionClose", GTK_STOCK_CLOSE, "_Close connection", NULL, "Close this connection", G_CALLBACK (connection_close_cb)},
         { "Quit", GTK_STOCK_QUIT, "_Quit", NULL, "Quit", G_CALLBACK (quit_cb)},
         { "Edit", NULL, "_Edit", NULL, "Edit", NULL },
+        { "Display", NULL, "_Display", NULL, "Display", NULL },
         { "Perspective", NULL, "_Perspective", NULL, "Perspective", NULL },
         { "Window", NULL, "_Window", NULL, "Window", NULL },
         { "WindowNew", STOCK_NEW_WINDOW, "_New window", "<control>N", "Open a new window for current connection", G_CALLBACK (window_new_cb)},
@@ -274,6 +299,8 @@ static const gchar *ui_actions_info =
         "    </menu>"
         "    <menu name='Edit' action='Edit'>"
         "    </menu>"
+        "    <menu name='Display' action='Display'>"
+        "    </menu>"
         "    <menu name='Perspective' action='Perspective'>"
         "      <placeholder name='PersList'/>"
         "    </menu>"
@@ -383,8 +410,12 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 	gtk_window_add_accel_group (GTK_WINDOW (bwin), accel_group);
 
         menubar = gtk_ui_manager_get_widget (ui, "/MenuBar");
+#ifdef HAVE_MAC_INTEGRATION
+	gtk_osxapplication_set_menu_bar (theApp, GTK_MENU_SHELL (menubar));
+#else
         gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
         gtk_widget_show (menubar);
+#endif
 
         toolbar = gtk_ui_manager_get_widget (ui, "/ToolBar");
         gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, TRUE, 0);
@@ -449,6 +480,12 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 		bwin->priv->ui_manager_merge_id = gtk_ui_manager_add_ui_from_string (bwin->priv->ui_manager,
 										     ui_info, -1, NULL);
 	bwin->priv->current_perspective = pers;
+	browser_perspective_get_current_customization (BROWSER_PERSPECTIVE (pers->perspective_widget),
+						       &actions, &ui_info);
+	browser_window_customize_perspective_ui (bwin, BROWSER_PERSPECTIVE (pers->perspective_widget),
+						 actions, ui_info);
+	if (actions)
+		g_object_unref (actions);
 	
 	/* insert perspective into window */
         bwin->priv->perspectives_nb = (GtkNotebook*) gtk_notebook_new ();
@@ -514,6 +551,18 @@ browser_window_new (BrowserConnection *bcnc, BrowserPerspectiveFactory *factory)
 
         gtk_widget_show (GTK_WIDGET (bwin));
 
+#if GTK_CHECK_VERSION(2,18,0)
+	gtk_widget_set_can_focus ((GtkWidget* )pers->perspective_widget, TRUE);
+#else
+	GtkWidget *fwid = (GtkWidget* )pers->perspective_widget;
+	if (! GTK_WIDGET_CAN_FOCUS (fwid)) {
+		GTK_WIDGET_SET_FLAGS (fwid, GTK_CAN_FOCUS);
+		gtk_widget_queue_resize (fwid);
+		g_object_notify (G_OBJECT (fwid), "can-focus");
+	}
+#endif
+	gtk_widget_grab_focus ((GtkWidget* )pers->perspective_widget);
+
 	return bwin;
 }
 
@@ -552,7 +601,7 @@ perspective_toggle_cb (GtkRadioAction *action, GtkRadioAction *current, BrowserW
 		bwin->priv->perspectives = g_slist_prepend (bwin->priv->perspectives, pers);
 		pers->page_number = gtk_notebook_append_page (bwin->priv->perspectives_nb,
 							      GTK_WIDGET (pers->perspective_widget), NULL);
-		gtk_widget_show_all ((GtkWidget*) bwin->priv->perspectives_nb);
+		gtk_widget_show (GTK_WIDGET (pers->perspective_widget));
 
 		GtkActionGroup *actions;
 		actions = browser_perspective_get_actions_group (BROWSER_PERSPECTIVE (pers->perspective_widget));
@@ -578,10 +627,14 @@ perspective_toggle_cb (GtkRadioAction *action, GtkRadioAction *current, BrowserW
 
 	/* current perspective's customizations */
 	bwin->priv->current_perspective = pers;
-	if (pers->customized_ui)
-		pers->customized_merge_id = gtk_ui_manager_add_ui_from_string (bwin->priv->ui_manager,
-									       pers->customized_ui,
-									       -1, NULL);
+
+	GtkActionGroup *actions;
+	browser_perspective_get_current_customization (BROWSER_PERSPECTIVE (pers->perspective_widget),
+						       &actions, &ui_info);
+	browser_window_customize_perspective_ui (bwin, BROWSER_PERSPECTIVE (pers->perspective_widget),
+						 actions, ui_info);
+	if (actions)
+		g_object_unref (actions);
 }
 
 static void
@@ -842,12 +895,29 @@ window_close_cb (GtkAction *action, BrowserWindow *bwin)
 static void
 window_fullscreen_cb (GtkToggleAction *action, BrowserWindow *bwin)
 {
-	if (gtk_toggle_action_get_active (action))
+	if (gtk_toggle_action_get_active (action)) {
 		gtk_window_fullscreen (GTK_WINDOW (bwin));
+		browser_show_notice (GTK_WINDOW (bwin),
+				     "fullscreen-esc",
+				     _("Hit the Escape key\nto leave the fullscreen mode"));
+	}
 	else
 		gtk_window_unfullscreen (GTK_WINDOW (bwin));
 }
 
+static gboolean
+key_press_event (GtkWidget *widget, GdkEventKey *event)
+{
+	if ((event->keyval == GDK_Escape) &&
+	    browser_window_is_fullscreen (BROWSER_WINDOW (widget))) {
+		browser_window_set_fullscreen (BROWSER_WINDOW (widget), FALSE);
+		return TRUE;
+	}
+
+	/* parent class */
+	return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+}
+
 static gboolean
 window_state_event (GtkWidget *widget, GdkEventWindowState *event)
 {
@@ -866,13 +936,16 @@ window_state_event (GtkWidget *widget, GdkEventWindowState *event)
 		wid = gtk_ui_manager_get_widget (bwin->priv->ui_manager, "/ToolBar");
 
                 fullscreen = event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN;
+		bwin->priv->fullscreen = fullscreen;
 		if (fullscreen) {
 			gtk_toolbar_set_style (GTK_TOOLBAR (wid), GTK_TOOLBAR_ICONS);
-			browser_spinner_set_size (BROWSER_SPINNER (bwin->priv->spinner), GTK_ICON_SIZE_LARGE_TOOLBAR);
+			browser_spinner_set_size (BROWSER_SPINNER (bwin->priv->spinner),
+						  GTK_ICON_SIZE_LARGE_TOOLBAR);
 		}
 		else {
 			gtk_toolbar_set_style (GTK_TOOLBAR (wid), bwin->priv->toolbar_style);
-			browser_spinner_set_size (BROWSER_SPINNER (bwin->priv->spinner), GTK_ICON_SIZE_DIALOG);
+			browser_spinner_set_size (BROWSER_SPINNER (bwin->priv->spinner),
+						  GTK_ICON_SIZE_SMALL_TOOLBAR);
 		}
 
 		wid = gtk_ui_manager_get_widget (bwin->priv->ui_manager, "/MenuBar");
@@ -882,6 +955,7 @@ window_state_event (GtkWidget *widget, GdkEventWindowState *event)
 			gtk_widget_show (wid);
 		
 		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (bwin->priv->statusbar), !fullscreen);
+		g_signal_emit (G_OBJECT (bwin), browser_window_signals[FULLSCREEN_CHANGED], 0, fullscreen);
         }
 	return FALSE;
 }
@@ -1281,3 +1355,33 @@ browser_window_change_perspective (BrowserWindow *bwin, const gchar *perspective
 
 	return bpers;
 }
+
+/**
+ * browser_window_is_fullscreen
+ * @bwin: a #BrowserWindow
+ *
+ * Returns: %TRUE if @bwin is fullscreen
+ */
+gboolean
+browser_window_is_fullscreen (BrowserWindow *bwin)
+{
+	g_return_val_if_fail (BROWSER_IS_WINDOW (bwin), FALSE);
+	return bwin->priv->fullscreen;
+}
+
+/**
+ * browser_window_set_fullscreen
+ * @bwin: a #BrowserWindow
+ * @fullscreen:
+ *
+ * Requires @bwin to be fullscreen if @fullscreen is %TRUE
+ */
+void
+browser_window_set_fullscreen (BrowserWindow *bwin, gboolean fullscreen)
+{
+	GtkAction *action;
+	g_return_if_fail (BROWSER_IS_WINDOW (bwin));
+	
+	action = gtk_action_group_get_action (bwin->priv->agroup, "WindowFullScreen");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), fullscreen);
+}
