@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 The GNOME Foundation
+ * Copyright (C) 2009 - 2010 The GNOME Foundation
  *
  * AUTHORS:
  *      Vivien Malerba <malerba@gnome-db.org>
@@ -208,9 +208,7 @@ static gboolean tree_store_drag_can_drag_cb (GdauiTreeStore *store, const gchar
 					     FavoriteSelector *tsel);
 static gboolean tree_store_drag_get_cb (GdauiTreeStore *store, const gchar *path,
 					GtkSelectionData *selection_data, FavoriteSelector *tsel);
-static void trash_data_received_cb (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
-				    GtkSelectionData *selection_data, guint target_type, guint time,
-				    FavoriteSelector *tsel);
+
 /**
  * favorite_selector_new
  *
@@ -264,7 +262,7 @@ favorite_selector_new (BrowserConnection *bcnc)
 				      G_TYPE_STRING, MGR_FAVORITES_CONTENTS_ATT_NAME,
 				      G_TYPE_UINT, MGR_FAVORITES_TYPE_ATT_NAME,
 				      G_TYPE_INT, MGR_FAVORITES_ID_ATT_NAME);
-	treeview = gtk_tree_view_new_with_model (model);
+	treeview = browser_make_tree_view (model);
 	tsel->priv->treeview = treeview;
 	g_object_unref (model);
 
