@@ -33,6 +33,9 @@ GtkOSXApplication *theApp = NULL;
 #endif
 
 /**
+ * browser_connection_open
+ * @error: a place to store errors, or %NULL
+ * 
  * Display a login dialog and, if validated, create a new #BrowserConnection
  *
  * Returns: a new #BrowserConnection, or %NULL (the caller DOES NOT OWN a reference to the returned value)
@@ -59,6 +62,10 @@ browser_connection_open (GError **error)
 }
 
 /**
+ * browser_connection_close
+ * @parent: a #GtkWindow
+ * @bcnc: a #BrowserConnection object
+ *
  * Displays a warning dialog and close @bcnc
  *
  * Returns: %TRUE if the connection has been closed
@@ -123,6 +130,7 @@ browser_show_error (GtkWindow *parent, const gchar *format, ...)
         va_list args;
         gchar sz[2048];
         GtkWidget *dialog;
+	gchar *tmp;
 
         /* build the message string */
         va_start (args, format);
@@ -130,11 +138,13 @@ browser_show_error (GtkWindow *parent, const gchar *format, ...)
         va_end (args);
 
         /* create the error message dialog */
-	dialog = gtk_message_dialog_new_with_markup (parent,
-                                                     GTK_DIALOG_DESTROY_WITH_PARENT |
-                                                     GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
-                                                     GTK_BUTTONS_CLOSE,
-						     "<span weight=\"bold\">%s</span>\n%s", _("Error:"), sz);
+	dialog = gtk_message_dialog_new (parent,
+					 GTK_DIALOG_DESTROY_WITH_PARENT |
+					 GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_CLOSE, NULL);
+	tmp = g_strdup_printf ("<span weight=\"bold\">%s</span>\n%s", _("Error:"), sz);
+	gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), tmp);
+	g_free (tmp);
 
         gtk_widget_show_all (dialog);
         gtk_dialog_run (GTK_DIALOG (dialog));
@@ -303,6 +313,8 @@ browser_show_help (GtkWindow *parent, const gchar *topic)
 		
 		g_error_free (error);
 	}
+	else
+		browser_show_notice (parent, "show-help", _("Help is being loaded, please wait..."));
 
 	g_free (uri);
 }
@@ -410,7 +422,9 @@ browser_get_pixbuf_icon (BrowserIconType type)
 		"gda-browser-column-nn.png",
 		"gda-browser-reference.png",
 		"gda-browser-diagram.png",
-		"gda-browser-query.png"
+		"gda-browser-query.png",
+		"gda-browser-grid.png",
+		"gda-browser-form.png",
 	};
 
 	if (!array)
@@ -518,6 +532,17 @@ connection_removed_cb (BrowserCore *bcore, BrowserConnection *bcnc, GdaDataModel
 	}
 }
 
+/**
+ * browser_make_small_button
+ * @is_toggle:
+ * @label:
+ * @stock_id:
+ * @tooltip:
+ *
+ * Creates a small button
+ *
+ * Returns: a new #GtkWidget
+ */
 GtkWidget *
 browser_make_small_button (gboolean is_toggle, const gchar *label, const gchar *stock_id, const gchar *tooltip)
 {
@@ -545,6 +570,7 @@ browser_make_small_button (gboolean is_toggle, const gchar *label, const gchar *
 	if (label) {
 		GtkWidget *wid;
 		wid = gtk_label_new (label);
+		gtk_misc_set_alignment (GTK_MISC (wid), 0., -1);
 		if (hbox)
 			gtk_box_pack_start (GTK_BOX (hbox), wid, FALSE, FALSE, 5);
 		else
@@ -556,3 +582,49 @@ browser_make_small_button (gboolean is_toggle, const gchar *label, const gchar *
 		gtk_widget_set_tooltip_text (button, tooltip);
 	return button;
 }
+
+static gboolean
+tree_view_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
+{
+	GtkTreeView *tree_view;
+	GtkTreeSelection *selection;
+
+	if (event->button != 3)
+		return FALSE;
+
+	tree_view = GTK_TREE_VIEW (widget);
+	selection = gtk_tree_view_get_selection (tree_view);
+
+	/* force selection of row on which clicked occurred */
+	GtkTreePath *path;
+	if ((event->window == gtk_tree_view_get_bin_window (tree_view)) &&
+	    gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y, &path, NULL, NULL, NULL)) {
+		gtk_tree_selection_unselect_all (selection);
+		gtk_tree_selection_select_path (selection, path);
+		gtk_tree_path_free (path);
+	}
+
+	return FALSE;
+}
+
+/**
+ * browser_make_tree_view
+ * @model: a #GtkTreeModel
+ *
+ * Creates a #GtkTreeView which, when right clicked, selects the row underneath the mouse
+ * cursor.
+ *
+ * Returns: a new #GtkWidget
+ */
+GtkWidget *
+browser_make_tree_view (GtkTreeModel *model)
+{
+	GtkWidget *tv;
+	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), NULL);
+	tv = gtk_tree_view_new_with_model (model);
+
+	g_signal_connect (G_OBJECT (tv), "button-press-event",
+                          G_CALLBACK (tree_view_button_pressed_cb), NULL);
+
+	return tv;
+}
